﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Threading;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Data;
using Dapfor.Wpf.Utils;

namespace Dapfor.Wpf.Grid.PerfDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow
    {
        private readonly ThreadSafeBindingList<TestClass> _bindingList = new ThreadSafeBindingList<TestClass>();
        private readonly DispatcherTimer _timer;
        private DateTime _lastTick = DateTime.MinValue;
        private long _processedUpdates;
        readonly Random _random = new Random();

        private readonly Brush _red;
        private readonly Brush _green;

        public MainWindow()
        {
            InitializeComponent();

            _red = new RadialGradientBrush(new GradientStopCollection
            {
                new GradientStop(Color.FromArgb(0, 255, 0, 0), 1), 
                new GradientStop(Color.FromArgb(100, 255, 0, 0), .6), 
                new GradientStop(Color.FromArgb(155, 255, 0, 0), 0)
            });

            _green = new RadialGradientBrush(new GradientStopCollection
            {
                new GradientStop(Color.FromArgb(0, 0, 255, 0), 1), 
                new GradientStop(Color.FromArgb(100, 0, 255, 0), .6), 
                new GradientStop(Color.FromArgb(155, 0, 255, 0), 0)
            });


            grid.ItemsSource = _bindingList;

            grid.RowUpdated += (sender, args) =>
            {
                Cell cell = args.Row[args.DataField.Id];
                cell.Blink(TimeSpan.FromMilliseconds(1000), _random.Next() %2 == 0 ? _red : _green);
            };

            _timer = new DispatcherTimer();
            _timer.Tick += OnTimerTick;
            _timer.Interval = TimeSpan.FromMilliseconds(10);
            _timer.Start();

            OnAdd5000ItemsClick(this, new RoutedEventArgs());
        }

        private void OnTimerTick(object sender, EventArgs e)
        {
            if (_bindingList.Count > 0)
            {
                //Update multiple objects at each timer tick
                int multiplier = (int)trackBar.Value;
                for (int i = 0; i < multiplier; ++i)
                {
                    _processedUpdates++;
                    TestClass testClass = _bindingList[_random.Next(_bindingList.Count)];
                    testClass.UpdateRandomValue();
                }
            }

            //Print the statistics each second
            if ((DateTime.UtcNow - _lastTick) > TimeSpan.FromSeconds(1))
            {
                lbVisibleSize.Text = string.Format("Grid visible size: {0:0}x{1:0}", grid.ActualWidth, grid.ActualHeight);
                lbTotalUpdates.Text = string.Format("Total updates/sec: {0}", _processedUpdates);

                Row topVisibleRow = grid.Rows.TopVisibleRow;
                Row bottomVisibleRow = grid.Rows.BottomVisibleRow;
                long visibleUpdates = 0;
                if (topVisibleRow != null && bottomVisibleRow != null && grid.Rows.Count > 0)
                {
                    visibleUpdates = _processedUpdates * (bottomVisibleRow.VisibleIndex - topVisibleRow.VisibleIndex) / grid.Rows.Count;
                }
                lbVisibleUpdates.Text = string.Format("Visible updates/sec: {0}", visibleUpdates);


                //Reset stat counters
                _lastTick = DateTime.UtcNow;
                _processedUpdates = 0;
            }
        }

        private void OnAdd5000ItemsClick(object sender, RoutedEventArgs e)
        {
            for (int i = 0; i < 5000; ++i)
            {
                _bindingList.Add(new TestClass());
            }
        }

        private void OnCbFilterCheckChanged(object sender, RoutedEventArgs e)
        {
            if(cbFilter.IsChecked == true)
            {
                //Create an easy filter
                grid.Filter = new Filter(delegate(Row row)
                {
                    var testClass = (TestClass)row.DataObject;
                    bool filtered = testClass.Value1 > 800000 ||
                                    testClass.Value2 > 800000 ||
                                    testClass.Value3 > 800000 ||
                                    testClass.Value4 > 800000 ||
                                    testClass.Value5 > 800000 ||
                                    testClass.Value6 > 800000 ||
                                    testClass.Value7 > 800000 ||
                                    testClass.Value8 > 800000 ||
                                    testClass.Value9 > 800000 ||
                                    testClass.Value10 > 800000;
                    return filtered;
                });
            }
            else
            {
                //Remove existing filter
                grid.Filter = null;
            }
        }
    }
}
